''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' View Portal2.bas '
'''''''''''''''''''
'
' Written by Justin Richards (2012)
'
'
' Remember the old Super Nintendo and MegaDrive games that had a Top Down type
' view of the map and you would scroll around in all directions as you walked
' along?  This program does just that!  The focus is not on graphics, just the
' engine itself, although more complex graphics can easily be added at a later
' stage.
'
' Basically the program spawns the player into the level and repositions him
' based on his position within the map.  When you move close to the edge of
' the playable area (called the view portal) you detach from the centre of the
' screen and move freely until you reach the edge.  This lets the player know
' that they have reached the edge and there is nothing else further in that
' direction.
'
' The source code is free for anyone to use.  Try playing around with the
' ManX and ManY variables and see what happens.  You could also try increasing
' the View Portal and adding more tiles.
'
' Enjoy...


TYPE GraphicsType

  PX AS STRING * 2 '  Pixel X Co-ordinate
  PY AS STRING * 2 '  Pixel Y Co-ordinate
  PCO AS STRING * 3 ' Pixel Colour Value

END TYPE


DIM SHARED GTIncrement
DIM SHARED GT AS GraphicsType

DIM SHARED BlueFloor(51) AS INTEGER
DIM SHARED GreenRoof(51) AS INTEGER

DIM SHARED GreenBrickTop(451) AS INTEGER
DIM SHARED GreenBrickBottom(451) AS INTEGER
DIM SHARED GreenBrickLeft(451) AS INTEGER
DIM SHARED GreenBrickRight(451) AS INTEGER

DIM SHARED GreenBrickTopLeft(451) AS INTEGER
DIM SHARED GreenBrickTopRight(451) AS INTEGER
DIM SHARED GreenBrickBottomLeft(451) AS INTEGER
DIM SHARED GreenBrickBottomRight(451) AS INTEGER

DIM SHARED GreenBrickTopLeftIn(451) AS INTEGER
DIM SHARED GreenBrickTopRightIn(451) AS INTEGER
DIM SHARED GreenBrickBottomLeftIn(451) AS INTEGER
DIM SHARED GreenBrickBottomRightIn(451) AS INTEGER


DIM SHARED BrownTileTop(51) AS INTEGER
DIM SHARED BrownTileBottom(51) AS INTEGER
DIM SHARED BrownTileLeft(51) AS INTEGER
DIM SHARED BrownTileRight(51) AS INTEGER

DIM SHARED BrownTileTopLeft(51) AS INTEGER
DIM SHARED BrownTileTopRight(51) AS INTEGER
DIM SHARED BrownTileBottomLeft(51) AS INTEGER
DIM SHARED BrownTileBottomRight(51) AS INTEGER

DIM SHARED BrownTileTopLeftIn(51) AS INTEGER
DIM SHARED BrownTileTopRightIn(51) AS INTEGER
DIM SHARED BrownTileBottomLeftIn(51) AS INTEGER
DIM SHARED BrownTileBottomRightIn(51) AS INTEGER

DECLARE SUB GetImages ' Import .gfx files into memory


' "Mask"           - Draw block frame work for determining accessable areas
' "BackGround"     - Draw the Background Tiles Only
' "Man"            - Draw the Man and Screen Border

DECLARE SUB DrawImages
DIM SHARED DrawWhat AS STRING


' Timers

DIM SHARED GameTimer

DIM SHARED TimeCount

DIM SHARED Secs, Mins


' Tiles

DIM SHARED TileType(200)
DIM SHARED TileProperty1(200)
DIM SHARED TileProperty2(200)
DIM SHARED TileProperty3(200)
DIM SHARED TileProperty4(200)
DIM SHARED TileX1(200)
DIM SHARED TileY1(200)
DIM SHARED TileX2(200)
DIM SHARED TileY2(200)



DIM SHARED ViewX1, ViewY1, ViewX2, ViewY2
DIM SHARED ViewX1Limit, ViewY1Limit, ViewX2Limit, ViewY2Limit

DIM SHARED ManX, ManY



IF _DIREXISTS("View Portal 2") THEN CHDIR "View Portal 2"


GTIncrement = 7 ' The length of our Graphics Type


CreatePalette

GetImages


GameTimer = -1
TimeCount = 1

FOR ResetTile = 0 TO 199
  TileType(ResetTile) = -1
  TileProperty1(ResetTile) = -1
  TileProperty2(ResetTile) = -1
  TileProperty3(ResetTile) = -1
  TileProperty4(ResetTile) = -1
  TileX1(ResetTile) = -1
  TileY1(ResetTile) = -1
  TileX2(ResetTile) = -1
  TileY2(ResetTile) = -1
NEXT ResetTile


' Call the gsGetTiles GOSUB routine to create the game objects.

GOSUB gsGetTiles




' Set Screen Limits (this example is an area 800 pixels x 800 pixels)

ViewX1Limit = -10: ViewY1Limit = 10: ViewX2Limit = 780: ViewY2Limit = 800


' Set starting co-ordinates for our Man (in this case he is just a green ball)

ManX = 110: ManY = 206


' Set the Veiw Portal to centre on the Man's co-ordinates to start with

ViewX1 = ManX - 160: ViewY1 = ManY - 100: ViewX2 = ManX + 159: ViewY2 = ManY + 99


' Test to see whether the man is close to the edge of the portal area and
' adjust the view portal position if necessary...

' X co-ordinates

IF ViewX1 < ViewX1Limit THEN

  ViewX1 = ViewX1Limit

  ViewX2 = ViewX1 + 319

ELSEIF ViewX2 < ViewX2Limit - 1 THEN

  ViewX1 = ViewX1Limit + 160 - ManX
  ViewX2 = ViewX1 + 319

ELSEIF ViewX2 > ViewX2Limit - 1 THEN

  ViewX1 = ViewX2Limit - 320

  ViewX1 = -ViewX1

  ViewX2 = ViewX1 + 319

END IF


' Y co-ordinates

IF ViewY1 < ViewY1Limit THEN

  ViewY1 = ViewY1Limit

  ViewY2 = ViewY1 + 199

ELSEIF ViewY2 < ViewY2Limit - 1 THEN

  ViewY1 = ViewY1Limit + 100 - ManY
  ViewY2 = ViewY1 + 199

ELSEIF ViewY2 > ViewY2Limit - 1 THEN

  ViewY1 = ViewY2Limit - 200

  ViewY1 = -ViewY1

  ViewY2 = ViewY1 + 199

END IF


' If the man is not near the edge than change his co-ordinates based on the
' new screen position...

IF ManX > ViewX1Limit + 160 AND ManX < ViewX2Limit - 160 THEN ManX = -ViewX1 + 160
IF ManY > ViewY1Limit + 100 AND ManY < ViewY2Limit - 100 THEN ManY = -ViewY1 + 100


' Set Keyboard Constants

CONST Key_Up& = 18432
CONST Key_Down& = 20480
CONST Key_Left& = 19200
CONST Key_Right& = 19712


' Set delay/speed variables

TimeSet = 1

GameSpeed = 0



' Main Game Loop

DO


  ' Reset the Tick Clock

  DEF SEG = 0

  POKE 1132, 0


  ' Only process the loop if the desired delay has passed

  IF GameSpeed = TimeSet THEN


    ' Exit code

    FOR KbdLoop = 0 TO 32

      Kbd$ = ""

      Kbd$ = INKEY$

      IF Kbd$ = CHR$(27) THEN

        CLS

        _DISPLAY

        SLEEP 1

        _FULLSCREEN _OFF

        SYSTEM

      END IF

    NEXT KbdLoop


    ' Draw the Mask

    LINE (ViewX1Limit + ViewX1, ViewY1Limit + ViewY1)-(ViewX2Limit + ViewX1, ViewY2Limit + ViewY1), 0, BF

    DrawWhat$ = "Mask"

    DrawImages



    ' Test for collisions

    PManUpTopLeft = POINT(ManX - 20 + ViewX1, ManY - 21 + ViewY1)
    PManUpTopMiddle = POINT(ManX + ViewX1, ManY - 21 + ViewY1)
    PManUpTopRight = POINT(ManX + 19 + ViewX1, ManY - 21 + ViewY1)

    PManDownBottomLeft = POINT(ManX - 20 + ViewX1, ManY + 20 + ViewY1)
    PManDownBottomMiddle = POINT(ManX + ViewX1, ManY + 20 + ViewY1)
    PManDownBottomRight = POINT(ManX + 19 + ViewX1, ManY + 20 + ViewY1)

    PManLeftTopLeft = POINT(ManX - 21 + ViewX1, ManY - 20 + ViewY1)
    PManLeftMiddleLeft = POINT(ManX - 21 + ViewX1, ManY + ViewY1)
    PManLeftBottomLeft = POINT(ManX - 21 + ViewX1, ManY + 19 + ViewY1)

    PManRightTopRight = POINT(ManX + 20 + ViewX1, ManY - 20 + ViewY1)
    PManRightMiddleRight = POINT(ManX + 20 + ViewX1, ManY + ViewY1)
    PManRightBottomRight = POINT(ManX + 20 + ViewX1, ManY + 19 + ViewY1)


    ' Draw the BackGround

    DrawWhat$ = "Background"

    DrawImages



    ' Test for keypresses

    UpOn = _KEYDOWN(Key_Up&)
    DownOn = _KEYDOWN(Key_Down&)
    LeftOn = _KEYDOWN(Key_Left&)
    RightOn = _KEYDOWN(Key_Right&)


    ' Move man based on user input

    IF UpOn = -1 AND DownOn = 0 AND PManUpTopLeft = 254 AND PManUpTopMiddle = 254 AND PManUpTopRight = 254 THEN

      IF ManY > ViewY1Limit + 100 AND ManY < ViewY2Limit - 98 THEN ViewY1 = ViewY1 + 2: ViewY2 = ViewY2 + 2

      ManY = ManY - 2

    END IF

    IF DownOn = -1 AND UpOn = 0 AND PManDownBottomLeft = 254 AND PManDownBottomMiddle = 254 AND PManDownBottomRight = 254 THEN

      IF ManY > ViewY1Limit + 98 AND ManY < ViewY2Limit - 100 THEN ViewY1 = ViewY1 - 2: ViewY2 = ViewY2 - 2

      ManY = ManY + 2

    END IF

    IF LeftOn = -1 AND RightOn = 0 AND PManLeftTopLeft = 254 AND PManLeftMiddleLeft = 254 AND PManLeftBottomLeft = 254 THEN

      IF ManX > ViewX1Limit + 160 AND ManX < ViewX2Limit - 158 THEN ViewX1 = ViewX1 + 2: ViewX2 = ViewX2 + 2

      ManX = ManX - 2

    END IF

    IF RightOn = -1 AND LeftOn = 0 AND PManRightTopRight = 254 AND PManRightMiddleRight = 254 AND PManRightBottomRight = 254 THEN

      IF ManX > ViewX1Limit + 158 AND ManX < ViewX2Limit - 160 THEN ViewX1 = ViewX1 - 2: ViewX2 = ViewX2 - 2

      ManX = ManX + 2

    END IF


    ' Draw the Man

    DrawWhat$ = "Man"

    DrawImages


    ' reset timer and display graphics

    GameSpeed = 0


    _DISPLAY

  END IF


  ' Loop here until an even amount of time has passed

  DO

  LOOP UNTIL PEEK(1132) >= 1

  DEF SEG


  ' Increment the timer

  GameSpeed = GameSpeed + 1


LOOP


END





gsGetTiles:

tc = 0 '                         Tile Count

TileType(tc) = 0 '                0 for Floor
TileX1(tc) = 60
TileY1(tc) = 150
TileX2(tc) = 119
TileY2(tc) = 239
tc = tc + 1

TileType(tc) = 0 '                0 for Floor
TileX1(tc) = 120
TileY1(tc) = 60
TileX2(tc) = 749
TileY2(tc) = 149
tc = tc + 1

TileType(tc) = 0 '                0 for Floor
TileX1(tc) = 120
TileY1(tc) = 150
TileX2(tc) = 209
TileY2(tc) = 659
tc = tc + 1

TileType(tc) = 0 '                0 for Floor
TileX1(tc) = 120
TileY1(tc) = 660
TileX2(tc) = 749
TileY2(tc) = 749
tc = tc + 1

TileType(tc) = 0 '                0 for Floor
TileX1(tc) = 570
TileY1(tc) = 150
TileX2(tc) = 749
TileY2(tc) = 659
tc = tc + 1


TileType(tc) = 1 '                1 for Wall
TileProperty1(tc) = 0 '           0 for Green Brick
TileProperty2(tc) = 2 '           2 for Left Side
TileX1(tc) = 30
TileY1(tc) = 150
TileX2(tc) = 59
TileY2(tc) = 239
tc = tc + 1

TileType(tc) = 1 '                1 for Wall
TileProperty1(tc) = 0 '           0 for Green Brick
TileProperty2(tc) = 2 '           2 for Left Side
TileX1(tc) = 90
TileY1(tc) = 60
TileX2(tc) = 119
TileY2(tc) = 119
tc = tc + 1

TileType(tc) = 1 '                1 for Wall
TileProperty1(tc) = 0 '           0 for Green Brick
TileProperty2(tc) = 2 '           2 for Left Side
TileX1(tc) = 90
TileY1(tc) = 270
TileX2(tc) = 119
TileY2(tc) = 749
tc = tc + 1

TileType(tc) = 1 '                1 for Wall
TileProperty1(tc) = 0 '           0 for Green Brick
TileProperty2(tc) = 2 '           2 for Left Side
TileX1(tc) = 540
TileY1(tc) = 180
TileX2(tc) = 569
TileY2(tc) = 629
tc = tc + 1


TileType(tc) = 1 '                1 for Wall
TileProperty1(tc) = 0 '           0 for Green Brick
TileProperty2(tc) = 0 '           0 for Top Side
TileX1(tc) = 120
TileY1(tc) = 30
TileX2(tc) = 749
TileY2(tc) = 59
tc = tc + 1

TileType(tc) = 1 '                1 for Wall
TileProperty1(tc) = 0 '           0 for Green Brick
TileProperty2(tc) = 0 '           0 for Top Side
TileX1(tc) = 60
TileY1(tc) = 120
TileX2(tc) = 89
TileY2(tc) = 149
tc = tc + 1

TileType(tc) = 1 '                1 for Wall
TileProperty1(tc) = 0 '           0 for Green Brick
TileProperty2(tc) = 0 '           0 for Top Side
TileX1(tc) = 240
TileY1(tc) = 630
TileX2(tc) = 539
TileY2(tc) = 659
tc = tc + 1



TileType(tc) = 1 '                1 for Wall
TileProperty1(tc) = 0 '           0 for Green Brick
TileProperty2(tc) = 1 '           1 for Bottom Side
TileX1(tc) = 60
TileY1(tc) = 240
TileX2(tc) = 89
TileY2(tc) = 269
tc = tc + 1

TileType(tc) = 1 '                1 for Wall
TileProperty1(tc) = 0 '           0 for Green Brick
TileProperty2(tc) = 1 '           1 for Bottom Side
TileX1(tc) = 120
TileY1(tc) = 750
TileX2(tc) = 749
TileY2(tc) = 779
tc = tc + 1

TileType(tc) = 1 '                1 for Wall
TileProperty1(tc) = 0 '           0 for Green Brick
TileProperty2(tc) = 1 '           1 for Bottom Side
TileX1(tc) = 240
TileY1(tc) = 150
TileX2(tc) = 539
TileY2(tc) = 179
tc = tc + 1


TileType(tc) = 1 '                1 for Wall
TileProperty1(tc) = 0 '           0 for Green Brick
TileProperty2(tc) = 3 '           3 for Right Side
TileX1(tc) = 750
TileY1(tc) = 60
TileX2(tc) = 779
TileY2(tc) = 749
tc = tc + 1

TileType(tc) = 1 '                1 for Wall
TileProperty1(tc) = 0 '           0 for Green Brick
TileProperty2(tc) = 3 '           3 for Right Side
TileX1(tc) = 210
TileY1(tc) = 180
TileX2(tc) = 239
TileY2(tc) = 629
tc = tc + 1



TileType(tc) = 1 '                1 for Wall
TileProperty1(tc) = 0 '           0 for Green Brick
TileProperty2(tc) = 4 '           4 for Top Left Side
TileX1(tc) = 30
TileY1(tc) = 120
TileX2(tc) = 59
TileY2(tc) = 149
tc = tc + 1

TileType(tc) = 1 '                1 for Wall
TileProperty1(tc) = 0 '           0 for Green Brick
TileProperty2(tc) = 4 '           4 for Top Left Side
TileX1(tc) = 90
TileY1(tc) = 30
TileX2(tc) = 119
TileY2(tc) = 59
tc = tc + 1

TileType(tc) = 1 '                1 for Wall
TileProperty1(tc) = 0 '           0 for Green Brick
TileProperty2(tc) = 6 '           6 for Bottom Left Side
TileX1(tc) = 30
TileY1(tc) = 240
TileX2(tc) = 59
TileY2(tc) = 269
tc = tc + 1

TileType(tc) = 1 '                1 for Wall
TileProperty1(tc) = 0 '           0 for Green Brick
TileProperty2(tc) = 6 '           6 for Bottom Left Side
TileX1(tc) = 90
TileY1(tc) = 750
TileX2(tc) = 119
TileY2(tc) = 779
tc = tc + 1

TileType(tc) = 1 '                1 for Wall
TileProperty1(tc) = 0 '           0 for Green Brick
TileProperty2(tc) = 5 '           5 for Top Right Side
TileX1(tc) = 750
TileY1(tc) = 30
TileX2(tc) = 779
TileY2(tc) = 59
tc = tc + 1

TileType(tc) = 1 '                1 for Wall
TileProperty1(tc) = 0 '           0 for Green Brick
TileProperty2(tc) = 7 '           7 for Bottom Right Side
TileX1(tc) = 750
TileY1(tc) = 750
TileX2(tc) = 779
TileY2(tc) = 779
tc = tc + 1

TileType(tc) = 1 '                1 for Wall
TileProperty1(tc) = 0 '           0 for Green Brick
TileProperty2(tc) = 11 '          11 for Bottom Right Side Inverted
TileX1(tc) = 90
TileY1(tc) = 120
TileX2(tc) = 119
TileY2(tc) = 149
tc = tc + 1

TileType(tc) = 1 '                1 for Wall
TileProperty1(tc) = 0 '           0 for Green Brick
TileProperty2(tc) = 9 '           9 for Top Right Side Inverted
TileX1(tc) = 90
TileY1(tc) = 240
TileX2(tc) = 119
TileY2(tc) = 269
tc = tc + 1

TileType(tc) = 1 '                1 for Wall
TileProperty1(tc) = 0 '           0 for Green Brick
TileProperty2(tc) = 8 '           8 for Top Left Side Inverted
TileX1(tc) = 210
TileY1(tc) = 150
TileX2(tc) = 239
TileY2(tc) = 179
tc = tc + 1

TileType(tc) = 1 '                1 for Wall
TileProperty1(tc) = 0 '           0 for Green Brick
TileProperty2(tc) = 9 '           9 for Top Right Side Inverted
TileX1(tc) = 540
TileY1(tc) = 150
TileX2(tc) = 569
TileY2(tc) = 179
tc = tc + 1

TileType(tc) = 1 '                1 for Wall
TileProperty1(tc) = 0 '           0 for Green Brick
TileProperty2(tc) = 10 '          10 for Bottom Left Side Inverted
TileX1(tc) = 210
TileY1(tc) = 630
TileX2(tc) = 239
TileY2(tc) = 659
tc = tc + 1

TileType(tc) = 1 '                1 for Wall
TileProperty1(tc) = 0 '           0 for Green Brick
TileProperty2(tc) = 11 '          11 for Bottom Right Side Inverted
TileX1(tc) = 540
TileY1(tc) = 630
TileX2(tc) = 569
TileY2(tc) = 659
tc = tc + 1



TileType(tc) = 2 '                1 for Floor Tile
TileProperty1(tc) = 0 '           0 for Brown
TileProperty2(tc) = 0 '           0 for Top Side
TileX1(tc) = 130
TileY1(tc) = 60
TileX2(tc) = 739
TileY2(tc) = 69
tc = tc + 1

TileType(tc) = 2 '                1 for Floor Tile
TileProperty1(tc) = 0 '           0 for Brown
TileProperty2(tc) = 0 '           0 for Top Side
TileX1(tc) = 210
TileY1(tc) = 660
TileX2(tc) = 569
TileY2(tc) = 669
tc = tc + 1

TileType(tc) = 2 '                1 for Floor Tile
TileProperty1(tc) = 0 '           0 for Brown
TileProperty2(tc) = 0 '           0 for Top Side
TileX1(tc) = 70
TileY1(tc) = 150
TileX2(tc) = 119
TileY2(tc) = 159
tc = tc + 1


TileType(tc) = 2 '                1 for Floor Tile
TileProperty1(tc) = 0 '           0 for Brown
TileProperty2(tc) = 1 '           1 for Bottom Side
TileX1(tc) = 130
TileY1(tc) = 740
TileX2(tc) = 739
TileY2(tc) = 749
tc = tc + 1

TileType(tc) = 2 '                1 for Floor Tile
TileProperty1(tc) = 0 '           0 for Brown
TileProperty2(tc) = 1 '           1 for Bottom Side
TileX1(tc) = 210
TileY1(tc) = 140
TileX2(tc) = 569
TileY2(tc) = 149
tc = tc + 1

TileType(tc) = 2 '                1 for Floor Tile
TileProperty1(tc) = 0 '           0 for Brown
TileProperty2(tc) = 1 '           1 for Bottom Side
TileX1(tc) = 70
TileY1(tc) = 230
TileX2(tc) = 119
TileY2(tc) = 239
tc = tc + 1


TileType(tc) = 2 '                1 for Floor Tile
TileProperty1(tc) = 0 '           0 for Brown
TileProperty2(tc) = 2 '           2 for Left Side
TileX1(tc) = 120
TileY1(tc) = 70
TileX2(tc) = 129
TileY2(tc) = 149
tc = tc + 1

TileType(tc) = 2 '                1 for Floor Tile
TileProperty1(tc) = 0 '           0 for Brown
TileProperty2(tc) = 2 '           2 for Left Side
TileX1(tc) = 120
TileY1(tc) = 240
TileX2(tc) = 129
TileY2(tc) = 739
tc = tc + 1

TileType(tc) = 2 '                1 for Floor Tile
TileProperty1(tc) = 0 '           0 for Brown
TileProperty2(tc) = 2 '           2 for Left Side
TileX1(tc) = 60
TileY1(tc) = 160
TileX2(tc) = 69
TileY2(tc) = 229
tc = tc + 1

TileType(tc) = 2 '                1 for Floor Tile
TileProperty1(tc) = 0 '           0 for Brown
TileProperty2(tc) = 2 '           2 for Left Side
TileX1(tc) = 570
TileY1(tc) = 150
TileX2(tc) = 579
TileY2(tc) = 659
tc = tc + 1


TileType(tc) = 2 '                1 for Floor Tile
TileProperty1(tc) = 0 '           0 for Brown
TileProperty2(tc) = 3 '           3 for Right Side
TileX1(tc) = 200
TileY1(tc) = 150
TileX2(tc) = 209
TileY2(tc) = 659
tc = tc + 1

TileType(tc) = 2 '                1 for Floor Tile
TileProperty1(tc) = 0 '           0 for Brown
TileProperty2(tc) = 3 '           3 for Right Side
TileX1(tc) = 740
TileY1(tc) = 70
TileX2(tc) = 749
TileY2(tc) = 739
tc = tc + 1


TileType(tc) = 2 '                1 for Floor Tile
TileProperty1(tc) = 0 '           0 for Brown
TileProperty2(tc) = 4 '           4 for Top Left Side
TileX1(tc) = 120
TileY1(tc) = 60
TileX2(tc) = 129
TileY2(tc) = 69
tc = tc + 1

TileType(tc) = 2 '                1 for Floor Tile
TileProperty1(tc) = 0 '           0 for Brown
TileProperty2(tc) = 4 '           4 for Top Left Side
TileX1(tc) = 60
TileY1(tc) = 150
TileX2(tc) = 69
TileY2(tc) = 159
tc = tc + 1

TileType(tc) = 2 '                1 for Floor Tile
TileProperty1(tc) = 0 '           0 for Brown
TileProperty2(tc) = 5 '           5 for Top Right Side
TileX1(tc) = 740
TileY1(tc) = 60
TileX2(tc) = 749
TileY2(tc) = 69
tc = tc + 1

TileType(tc) = 2 '                1 for Floor Tile
TileProperty1(tc) = 0 '           0 for Brown
TileProperty2(tc) = 6 '           6 for Bottom Left Side
TileX1(tc) = 120
TileY1(tc) = 740
TileX2(tc) = 129
TileY2(tc) = 749
tc = tc + 1

TileType(tc) = 2 '                1 for Floor Tile
TileProperty1(tc) = 0 '           0 for Brown
TileProperty2(tc) = 6 '           6 for Bottom Left Side
TileX1(tc) = 60
TileY1(tc) = 230
TileX2(tc) = 69
TileY2(tc) = 239
tc = tc + 1

TileType(tc) = 2 '                1 for Floor Tile
TileProperty1(tc) = 0 '           0 for Brown
TileProperty2(tc) = 7 '           7 for Bottom Right Side
TileX1(tc) = 740
TileY1(tc) = 740
TileX2(tc) = 749
TileY2(tc) = 749
tc = tc + 1


TileType(tc) = 2 '                1 for Floor Tile
TileProperty1(tc) = 0 '           0 for Brown
TileProperty2(tc) = 8 '           8 for Top Left Side Inverted
TileX1(tc) = 120
TileY1(tc) = 150
TileX2(tc) = 129
TileY2(tc) = 159
tc = tc + 1

TileType(tc) = 2 '                1 for Floor Tile
TileProperty1(tc) = 0 '           0 for Brown
TileProperty2(tc) = 8 '           8 for Top Left Side Inverted
TileX1(tc) = 570
TileY1(tc) = 660
TileX2(tc) = 579
TileY2(tc) = 669
tc = tc + 1

TileType(tc) = 2 '                1 for Floor Tile
TileProperty1(tc) = 0 '           0 for Brown
TileProperty2(tc) = 9 '           9 for Top Right Side Inverted
TileX1(tc) = 200
TileY1(tc) = 660
TileX2(tc) = 209
TileY2(tc) = 669
tc = tc + 1

TileType(tc) = 2 '                1 for Floor Tile
TileProperty1(tc) = 0 '           0 for Brown
TileProperty2(tc) = 10 '           10 for Bottom Left Side Inverted
TileX1(tc) = 570
TileY1(tc) = 140
TileX2(tc) = 579
TileY2(tc) = 149
tc = tc + 1

TileType(tc) = 2 '                1 for Floor Tile
TileProperty1(tc) = 0 '           0 for Brown
TileProperty2(tc) = 10 '           10 for Bottom Left Side Inverted
TileX1(tc) = 120
TileY1(tc) = 230
TileX2(tc) = 129
TileY2(tc) = 239
tc = tc + 1

TileType(tc) = 2 '                1 for Floor Tile
TileProperty1(tc) = 0 '           0 for Brown
TileProperty2(tc) = 11 '           11 for Bottom Right Side Inverted
TileX1(tc) = 200
TileY1(tc) = 140
TileX2(tc) = 209
TileY2(tc) = 149
tc = tc + 1


RETURN





SUB GetImages


_FULLSCREEN


OPEN "Resources\GreenRoof.gfx" FOR BINARY AS 1 LEN = LEN(GT): GOSUB gsGetLoop
GET (0, 0)-(9, 9), GreenRoof(): CLOSE #1

OPEN "Resources\BlueFloor.gfx" FOR BINARY AS 1 LEN = LEN(GT): GOSUB gsGetLoop
GET (0, 0)-(9, 9), BlueFloor(): CLOSE #1


OPEN "Resources\GreenBrickTop.gfx" FOR BINARY AS 1 LEN = LEN(GT): GOSUB gsGetLoop
GET (0, 0)-(29, 29), GreenBrickTop(): CLOSE #1

OPEN "Resources\GreenBrickBottom.gfx" FOR BINARY AS 1 LEN = LEN(GT): GOSUB gsGetLoop
GET (0, 0)-(29, 29), GreenBrickBottom(): CLOSE #1

OPEN "Resources\GreenBrickLeft.gfx" FOR BINARY AS 1 LEN = LEN(GT): GOSUB gsGetLoop
GET (0, 0)-(29, 29), GreenBrickLeft(): CLOSE #1

OPEN "Resources\GreenBrickRight.gfx" FOR BINARY AS 1 LEN = LEN(GT): GOSUB gsGetLoop
GET (0, 0)-(29, 29), GreenBrickRight(): CLOSE #1


OPEN "Resources\GreenBrickTopLeft.gfx" FOR BINARY AS 1 LEN = LEN(GT): GOSUB gsGetLoop
GET (0, 0)-(29, 29), GreenBrickTopLeft(): CLOSE #1

OPEN "Resources\GreenBrickTopRight.gfx" FOR BINARY AS 1 LEN = LEN(GT): GOSUB gsGetLoop
GET (0, 0)-(29, 29), GreenBrickTopRight(): CLOSE #1

OPEN "Resources\GreenBrickBottomLeft.gfx" FOR BINARY AS 1 LEN = LEN(GT): GOSUB gsGetLoop
GET (0, 0)-(29, 29), GreenBrickBottomLeft(): CLOSE #1

OPEN "Resources\GreenBrickBottomRight.gfx" FOR BINARY AS 1 LEN = LEN(GT): GOSUB gsGetLoop
GET (0, 0)-(29, 29), GreenBrickBottomRight(): CLOSE #1



OPEN "Resources\GreenBrickTopLeftIn.gfx" FOR BINARY AS 1 LEN = LEN(GT): GOSUB gsGetLoop
GET (0, 0)-(29, 29), GreenBrickTopLeftIn(): CLOSE #1

OPEN "Resources\GreenBrickTopRightIn.gfx" FOR BINARY AS 1 LEN = LEN(GT): GOSUB gsGetLoop
GET (0, 0)-(29, 29), GreenBrickTopRightIn(): CLOSE #1

OPEN "Resources\GreenBrickBottomLeftIn.gfx" FOR BINARY AS 1 LEN = LEN(GT): GOSUB gsGetLoop
GET (0, 0)-(29, 29), GreenBrickBottomLeftIn(): CLOSE #1

OPEN "Resources\GreenBrickBottomRightIn.gfx" FOR BINARY AS 1 LEN = LEN(GT): GOSUB gsGetLoop
GET (0, 0)-(29, 29), GreenBrickBottomRightIn(): CLOSE #1



OPEN "Resources\BrownTileTop.gfx" FOR BINARY AS 1 LEN = LEN(GT): GOSUB gsGetLoop
GET (0, 0)-(9, 9), BrownTileTop(): CLOSE #1

OPEN "Resources\BrownTileBottom.gfx" FOR BINARY AS 1 LEN = LEN(GT): GOSUB gsGetLoop
GET (0, 0)-(9, 9), BrownTileBottom(): CLOSE #1

OPEN "Resources\BrownTileLeft.gfx" FOR BINARY AS 1 LEN = LEN(GT): GOSUB gsGetLoop
GET (0, 0)-(9, 9), BrownTileLeft(): CLOSE #1

OPEN "Resources\BrownTileRight.gfx" FOR BINARY AS 1 LEN = LEN(GT): GOSUB gsGetLoop
GET (0, 0)-(9, 9), BrownTileRight(): CLOSE #1


OPEN "Resources\BrownTileTopLeft.gfx" FOR BINARY AS 1 LEN = LEN(GT): GOSUB gsGetLoop
GET (0, 0)-(9, 9), BrownTileTopLeft(): CLOSE #1

OPEN "Resources\BrownTileTopRight.gfx" FOR BINARY AS 1 LEN = LEN(GT): GOSUB gsGetLoop
GET (0, 0)-(9, 9), BrownTileTopRight(): CLOSE #1

OPEN "Resources\BrownTileBottomLeft.gfx" FOR BINARY AS 1 LEN = LEN(GT): GOSUB gsGetLoop
GET (0, 0)-(9, 9), BrownTileBottomLeft(): CLOSE #1

OPEN "Resources\BrownTileBottomRight.gfx" FOR BINARY AS 1 LEN = LEN(GT): GOSUB gsGetLoop
GET (0, 0)-(9, 9), BrownTileBottomRight(): CLOSE #1


OPEN "Resources\BrownTileTopLeftIn.gfx" FOR BINARY AS 1 LEN = LEN(GT): GOSUB gsGetLoop
GET (0, 0)-(9, 9), BrownTileTopLeftIn(): CLOSE #1

OPEN "Resources\BrownTileTopRightIn.gfx" FOR BINARY AS 1 LEN = LEN(GT): GOSUB gsGetLoop
GET (0, 0)-(9, 9), BrownTileTopRightIn(): CLOSE #1

OPEN "Resources\BrownTileBottomLeftIn.gfx" FOR BINARY AS 1 LEN = LEN(GT): GOSUB gsGetLoop
GET (0, 0)-(9, 9), BrownTileBottomLeftIn(): CLOSE #1

OPEN "Resources\BrownTileBottomRightIn.gfx" FOR BINARY AS 1 LEN = LEN(GT): GOSUB gsGetLoop
GET (0, 0)-(9, 9), BrownTileBottomRightIn(): CLOSE #1



LINE (0, 0)-(319, 199), 254, BF


'PCOPY 1, 0

EXIT SUB



gsGetLoop:


LoopIncrement = LOF(1)

GET #1, LoopIncrement - GTIncrement + 1, GT

PointX = VAL(GT.PX)
PointY = VAL(GT.PY)

LoopIncrement = -GTIncrement

FOR DrawY = 0 TO PointY

  FOR DrawX = 0 TO PointX

    GET #1, LoopIncrement + GTIncrement + 1, GT

    PointCo = VAL(GT.PCO)

    PSET (DrawX, DrawY), PointCo

    LoopIncrement = LoopIncrement + GTIncrement

  NEXT DrawX

NEXT DrawY


RETURN

END SUB





SUB DrawImages

IF DrawWhat$ = "Mask" THEN

  FOR DrawTile = 0 TO 199

    IF TileType(DrawTile) = 0 THEN LINE (TileX1(DrawTile) + ViewX1, TileY1(DrawTile) + ViewY1)-(TileX2(DrawTile) + ViewX1, TileY2(DrawTile) + ViewY1), 254, BF
    IF TileType(DrawTile) = 1 THEN LINE (TileX1(DrawTile) + ViewX1, TileY1(DrawTile) + ViewY1)-(TileX2(DrawTile) + ViewX1, TileY2(DrawTile) + ViewY1), 15, BF

  NEXT DrawTile

END IF


IF DrawWhat$ = "Background" THEN

  FOR DrawTile = 0 TO 199

    IF TileType(DrawTile) = 0 THEN GOSUB gsDrawFloor
    IF TileType(DrawTile) = 1 THEN GOSUB gsDrawWall
    IF TileType(DrawTile) = 2 THEN GOSUB gsDrawFloorTile

  NEXT DrawTile

  FOR DrawRoofY = ViewY1Limit + ViewY1 TO ViewY2Limit + ViewY1 STEP 10

    FOR DrawRoofX = ViewX1Limit + ViewX1 TO ViewX2Limit + ViewX1 STEP 10

      DrawRoofTile = POINT(DrawRoofX, DrawRoofY)

      IF DrawRoofTile = 0 THEN PUT (DrawRoofX, DrawRoofY), GreenRoof(), _CLIP PSET, 255

    NEXT DrawRoofX

  NEXT DrawRoofY

END IF


IF DrawWhat$ = "Man" THEN

  CIRCLE (ManX + ViewX1, ManY + ViewY1), 19, 55
  CIRCLE (ManX - 1 + ViewX1, ManY - 1 + ViewY1), 19, 55
  PAINT (ManX + ViewX1, ManY + ViewY1), 55, 55


  ' Draw Border around Screen Edge

  LINE (0, 0)-(319, 9), 254, BF
  LINE (0, 190)-(319, 199), 254, BF
  LINE (0, 10)-(9, 189), 254, BF
  LINE (310, 10)-(319, 189), 254, BF

END IF


EXIT SUB


gsDrawFloor:

FOR DrawFloorY = TileY1(DrawTile) TO TileY2(DrawTile) STEP 10

  FOR DrawFloorX = TileX1(DrawTile) TO TileX2(DrawTile) STEP 10

    PUT (DrawFloorX + ViewX1, DrawFloorY + ViewY1), BlueFloor(), _CLIP PSET, 255

  NEXT DrawFloorX

NEXT DrawFloorY

RETURN



gsDrawWall:

FOR DrawWallY = TileY1(DrawTile) TO TileY2(DrawTile) STEP 30

  FOR DrawWallX = TileX1(DrawTile) TO TileX2(DrawTile) STEP 30

    SELECT CASE TileProperty1(DrawTile)

      CASE 0 ' Green Brick

        IF TileProperty2(DrawTile) = 0 THEN PUT (DrawWallX + ViewX1, DrawWallY + ViewY1), GreenBrickTop(), _CLIP PSET, 255
        IF TileProperty2(DrawTile) = 1 THEN PUT (DrawWallX + ViewX1, DrawWallY + ViewY1), GreenBrickBottom(), _CLIP PSET, 255
        IF TileProperty2(DrawTile) = 2 THEN PUT (DrawWallX + ViewX1, DrawWallY + ViewY1), GreenBrickLeft(), _CLIP PSET, 255
        IF TileProperty2(DrawTile) = 3 THEN PUT (DrawWallX + ViewX1, DrawWallY + ViewY1), GreenBrickRight(), _CLIP PSET, 255
        IF TileProperty2(DrawTile) = 4 THEN PUT (DrawWallX + ViewX1, DrawWallY + ViewY1), GreenBrickTopLeft(), _CLIP PSET, 255
        IF TileProperty2(DrawTile) = 5 THEN PUT (DrawWallX + ViewX1, DrawWallY + ViewY1), GreenBrickTopRight(), _CLIP PSET, 255
        IF TileProperty2(DrawTile) = 6 THEN PUT (DrawWallX + ViewX1, DrawWallY + ViewY1), GreenBrickBottomLeft(), _CLIP PSET, 255
        IF TileProperty2(DrawTile) = 7 THEN PUT (DrawWallX + ViewX1, DrawWallY + ViewY1), GreenBrickBottomRight(), _CLIP PSET, 255
        IF TileProperty2(DrawTile) = 8 THEN PUT (DrawWallX + ViewX1, DrawWallY + ViewY1), GreenBrickTopLeftIn(), _CLIP PSET, 255
        IF TileProperty2(DrawTile) = 9 THEN PUT (DrawWallX + ViewX1, DrawWallY + ViewY1), GreenBrickTopRightIn(), _CLIP PSET, 255
        IF TileProperty2(DrawTile) = 10 THEN PUT (DrawWallX + ViewX1, DrawWallY + ViewY1), GreenBrickBottomLeftIn(), _CLIP PSET, 255
        IF TileProperty2(DrawTile) = 11 THEN PUT (DrawWallX + ViewX1, DrawWallY + ViewY1), GreenBrickBottomRightIn(), _CLIP PSET, 255

    END SELECT

  NEXT DrawWallX

NEXT DrawWallY


RETURN


gsDrawFloorTile:

FOR DrawWallY = TileY1(DrawTile) TO TileY2(DrawTile) STEP 10

  FOR DrawWallX = TileX1(DrawTile) TO TileX2(DrawTile) STEP 10

    SELECT CASE TileProperty1(DrawTile)

      CASE 0 ' Brown Tile

        IF TileProperty2(DrawTile) = 0 THEN PUT (DrawWallX + ViewX1, DrawWallY + ViewY1), BrownTileTop(), _CLIP PSET, 255
        IF TileProperty2(DrawTile) = 1 THEN PUT (DrawWallX + ViewX1, DrawWallY + ViewY1), BrownTileBottom(), _CLIP PSET, 255
        IF TileProperty2(DrawTile) = 2 THEN PUT (DrawWallX + ViewX1, DrawWallY + ViewY1), BrownTileLeft(), _CLIP PSET, 255
        IF TileProperty2(DrawTile) = 3 THEN PUT (DrawWallX + ViewX1, DrawWallY + ViewY1), BrownTileRight(), _CLIP PSET, 255
        IF TileProperty2(DrawTile) = 4 THEN PUT (DrawWallX + ViewX1, DrawWallY + ViewY1), BrownTileTopLeft(), _CLIP PSET, 255
        IF TileProperty2(DrawTile) = 5 THEN PUT (DrawWallX + ViewX1, DrawWallY + ViewY1), BrownTileTopRight(), _CLIP PSET, 255
        IF TileProperty2(DrawTile) = 6 THEN PUT (DrawWallX + ViewX1, DrawWallY + ViewY1), BrownTileBottomLeft(), _CLIP PSET, 255
        IF TileProperty2(DrawTile) = 7 THEN PUT (DrawWallX + ViewX1, DrawWallY + ViewY1), BrownTileBottomRight(), _CLIP PSET, 255
        IF TileProperty2(DrawTile) = 8 THEN PUT (DrawWallX + ViewX1, DrawWallY + ViewY1), BrownTileTopLeftIn(), _CLIP PSET, 255
        IF TileProperty2(DrawTile) = 9 THEN PUT (DrawWallX + ViewX1, DrawWallY + ViewY1), BrownTileTopRightIn(), _CLIP PSET, 255
        IF TileProperty2(DrawTile) = 10 THEN PUT (DrawWallX + ViewX1, DrawWallY + ViewY1), BrownTileBottomLeftIn(), _CLIP PSET, 255
        IF TileProperty2(DrawTile) = 11 THEN PUT (DrawWallX + ViewX1, DrawWallY + ViewY1), BrownTileBottomRightIn(), _CLIP PSET, 255


    END SELECT

  NEXT DrawWallX

NEXT DrawWallY



RETURN


END SUB




SUB CreatePalette

SCREEN _NEWIMAGE(320, 200, 256)


' Set Custom Palette...

' Gray Scale

RedStart = 3: GreenStart = 3: BlueStart = 3
Red = RedStart: Green = GreenStart: Blue = BlueStart

FOR PaletteLoop = 0 TO 15

  ColourValue = 65536 * Blue + 256 * Green + Red

  PALETTE PaletteLoop, ColourValue

  ColourIncrement = ColourIncrement + 4

  Red = RedStart + ColourIncrement: Green = GreenStart + ColourIncrement: Blue = BlueStart + ColourIncrement

  IF Red > 63 THEN Red = 63
  IF Green > 63 THEN Green = 63
  IF Blue > 63 THEN Blue = 63

NEXT PaletteLoop

ColourIncrement = 0


' Brown

RedStart = 4: GreenStart = 2: BlueStart = 1
Red = RedStart: Green = GreenStart: Blue = BlueStart

FOR PaletteLoop = 16 TO 31

  ColourValue = 65536 * Blue + 256 * Green + Red

  PALETTE PaletteLoop, ColourValue

  ColourIncrement = ColourIncrement + 2

  Red = RedStart + ColourIncrement * 3: Green = GreenStart + ColourIncrement * 2: Blue = BlueStart + ColourIncrement

  IF Red > 63 THEN Red = 63
  IF Green > 63 THEN Green = 63
  IF Blue > 63 THEN Blue = 63

NEXT PaletteLoop

ColourIncrement = 0



' Orange / Yellow

RedStart = 32: GreenStart = 16: BlueStart = 0
Red = RedStart: Green = GreenStart: Blue = BlueStart

FOR PaletteLoop = 32 TO 47

  ColourValue = 65536 * Blue + 256 * Green + Red

  PALETTE PaletteLoop, ColourValue

  ColourIncrement = ColourIncrement + 2

  Red = RedStart + ColourIncrement * 4: Green = GreenStart + ColourIncrement * 2

  IF Red > 63 THEN Red = 63
  IF Green > 63 THEN Green = 63

  IF Red = 63 AND Green = 63 THEN Blue = Blue + 10

  IF Blue > 63 THEN Blue = 63

NEXT PaletteLoop

ColourIncrement = 0



' Green

RedStart = 0: GreenStart = 3: BlueStart = 0
Red = RedStart: Green = GreenStart: Blue = BlueStart

FOR PaletteLoop = 48 TO 63

  ColourValue = 65536 * Blue + 256 * Green + Red

  PALETTE PaletteLoop, ColourValue

  ColourIncrement = ColourIncrement + 3

  Green = GreenStart + ColourIncrement * 2

  IF Green > 63 THEN Green = 63

  IF Green = 63 THEN
    Blue = Blue + 8
    Red = Red + 8
  END IF

  IF Red > 63 THEN Red = 63
  IF Blue > 63 THEN Blue = 63

NEXT PaletteLoop

ColourIncrement = 0



' Dark Blue

RedStart = 0: GreenStart = 0: BlueStart = 3
Red = RedStart: Green = GreenStart: Blue = BlueStart

FOR PaletteLoop = 64 TO 79

  ColourValue = 65536 * Blue + 256 * Green + Red

  PALETTE PaletteLoop, ColourValue

  ColourIncrement = ColourIncrement + 3

  Blue = BlueStart + ColourIncrement * 2
  Green = GreenStart + ColourIncrement / 1.5

  IF Blue > 63 THEN Blue = 63

  '  IF Blue >= 30 THEN Green = Green * 1.5

  IF Green > 63 THEN Green = 63

  '  IF Green >= 1 THEN Red = Red * 4

  IF Red > 63 THEN Red = 63

NEXT PaletteLoop

ColourIncrement = 0



' Light Blue

RedStart = 0: GreenStart = 20: BlueStart = 63
Red = RedStart: Green = GreenStart: Blue = BlueStart

FOR PaletteLoop = 80 TO 95

  ColourValue = 65536 * Blue + 256 * Green + Red

  PALETTE PaletteLoop, ColourValue

  ColourIncrement = ColourIncrement + 3

  Green = GreenStart + ColourIncrement
  Red = RedStart + ColourIncrement


  IF Green > 63 THEN Green = 63

  IF Red > 63 THEN Red = 63

NEXT PaletteLoop

ColourIncrement = 0



' Purple

RedStart = 6: GreenStart = 0: BlueStart = 6
Red = RedStart: Green = GreenStart: Blue = BlueStart

FOR PaletteLoop = 96 TO 111

  ColourValue = 65536 * Blue + 256 * Green + Red

  PALETTE PaletteLoop, ColourValue

  ColourIncrement = ColourIncrement + 4

  Blue = BlueStart + ColourIncrement
  Red = RedStart + ColourIncrement


  IF Blue > 63 THEN Blue = 63
  IF Red > 63 THEN Red = 63

  IF Blue > 30 AND Red > 30 THEN Green = Green + 4

  IF Green > 63 THEN Green = 63

NEXT PaletteLoop

ColourIncrement = 0



' Red

RedStart = 4: GreenStart = 0: BlueStart = 0
Red = RedStart: Green = GreenStart: Blue = BlueStart

FOR PaletteLoop = 112 TO 127

  ColourValue = 65536 * Blue + 256 * Green + Red

  PALETTE PaletteLoop, ColourValue

  ColourIncrement = ColourIncrement + 4

  Red = RedStart + ColourIncrement

  IF Red > 63 THEN Red = 63

NEXT PaletteLoop

ColourIncrement = 0



' Pale Pink

RedStart = 63: GreenStart = 0: BlueStart = 0
Red = RedStart: Green = GreenStart: Blue = BlueStart

FOR PaletteLoop = 128 TO 143

  ColourValue = 65536 * Blue + 256 * Green + Red

  PALETTE PaletteLoop, ColourValue

  ColourIncrement = ColourIncrement + 3

  Blue = BlueStart + ColourIncrement
  Green = GreenStart + ColourIncrement

  IF Blue > 63 THEN Blue = 63
  IF Green > 63 THEN Green = 63

NEXT PaletteLoop

ColourIncrement = 0



' Gun Metal Grey

RedStart = 4: GreenStart = 6: BlueStart = 6
Red = RedStart: Green = GreenStart: Blue = BlueStart

FOR PaletteLoop = 144 TO 159

  ColourValue = 65536 * Blue + 256 * Green + Red

  PALETTE PaletteLoop, ColourValue

  ColourIncrement = ColourIncrement + 4

  Red = RedStart + ColourIncrement / 1.5
  Blue = BlueStart + ColourIncrement * 1.5
  Green = GreenStart + ColourIncrement

  IF Red > 63 THEN Red = 63
  IF Blue > 63 THEN Blue = 63
  IF Green > 63 THEN Green = 63

NEXT PaletteLoop

ColourIncrement = 0



' Pale Green

RedStart = 4: GreenStart = 8: BlueStart = 4
Red = RedStart: Green = GreenStart: Blue = BlueStart

FOR PaletteLoop = 160 TO 175

  ColourValue = 65536 * Blue + 256 * Green + Red

  PALETTE PaletteLoop, ColourValue

  ColourIncrement = ColourIncrement + 4

  Red = RedStart + ColourIncrement / 1.2
  Blue = BlueStart + ColourIncrement
  Green = GreenStart + ColourIncrement * 1.5

  IF Red > 63 THEN Red = 63
  IF Blue > 63 THEN Blue = 63
  IF Green > 63 THEN Green = 63

NEXT PaletteLoop

ColourIncrement = 0



' Dark Purple

RedStart = 6: GreenStart = 0: BlueStart = 8
Red = RedStart: Green = GreenStart: Blue = BlueStart

FOR PaletteLoop = 176 TO 191

  ColourValue = 65536 * Blue + 256 * Green + Red

  PALETTE PaletteLoop, ColourValue

  ColourIncrement = ColourIncrement + 2

  Red = RedStart + ColourIncrement * 2
  Blue = BlueStart + ColourIncrement * 2
  Green = GreenStart + ColourIncrement * 1.5

  IF Red > 63 THEN Red = 63
  IF Blue > 63 THEN Blue = 63
  IF Green > 63 THEN Green = 63

NEXT PaletteLoop

ColourIncrement = 0



' Dark Maroon

RedStart = 5: GreenStart = 3: BlueStart = 3
Red = RedStart: Green = GreenStart: Blue = BlueStart

FOR PaletteLoop = 192 TO 207

  ColourValue = 65536 * Blue + 256 * Green + Red

  PALETTE PaletteLoop, ColourValue

  ColourIncrement = ColourIncrement + 2

  Red = RedStart + ColourIncrement * 2
  Blue = BlueStart + ColourIncrement * 1.5
  Green = GreenStart + ColourIncrement * 1.5

  IF Red > 63 THEN Red = 63
  IF Blue > 63 THEN Blue = 63
  IF Green > 63 THEN Green = 63

NEXT PaletteLoop

ColourIncrement = 0



' Dark Cyan

RedStart = 5: GreenStart = 8: BlueStart = 8
Red = RedStart: Green = GreenStart: Blue = BlueStart

FOR PaletteLoop = 208 TO 223

  ColourValue = 65536 * Blue + 256 * Green + Red

  PALETTE PaletteLoop, ColourValue

  ColourIncrement = ColourIncrement + 1.5

  Red = RedStart + ColourIncrement * 1.5
  Blue = BlueStart + ColourIncrement * 2
  Green = GreenStart + ColourIncrement * 2

  IF Red > 63 THEN Red = 63
  IF Blue > 63 THEN Blue = 63
  IF Green > 63 THEN Green = 63

NEXT PaletteLoop

ColourIncrement = 0




' Bronze

RedStart = 12: GreenStart = 10: BlueStart = 3
Red = RedStart: Green = GreenStart: Blue = BlueStart

FOR PaletteLoop = 224 TO 239

  ColourValue = 65536 * Blue + 256 * Green + Red

  PALETTE PaletteLoop, ColourValue

  ColourIncrement = ColourIncrement + 2

  Red = RedStart + ColourIncrement * 1.5
  Blue = BlueStart + ColourIncrement * 1
  Green = GreenStart + ColourIncrement * 1.5

  IF Red > 63 THEN Red = 63
  IF Blue > 63 THEN Blue = 63
  IF Green > 63 THEN Green = 63

NEXT PaletteLoop

ColourIncrement = 0



' Aqua Dark

RedStart = 0: GreenStart = 10: BlueStart = 9
Red = RedStart: Green = GreenStart: Blue = BlueStart

FOR PaletteLoop = 240 TO 253

  ColourValue = 65536 * Blue + 256 * Green + Red

  PALETTE PaletteLoop, ColourValue

  ColourIncrement = ColourIncrement + 3

  Blue = BlueStart + ColourIncrement * 1
  Green = GreenStart + ColourIncrement * 1

  IF Blue > 63 THEN Blue = 63
  IF Green > 63 THEN Green = 63

NEXT PaletteLoop

ColourIncrement = 0


' Set Solid & Transparent Colour


ColourValue = 65536 * 0 + 256 * 0 + 0

PALETTE 254, ColourValue


ColourValue = 65536 * 63 + 256 * 0 + 63

PALETTE 255, ColourValue



END SUB


